#ifndef PIRAT_H
#   define PIRAT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <math.h>
#include <unistd.h>
#include <float.h>
#if HAVE_STRING_H
#   include <string.h>
#else
#   if HAVE_STRINGS_H
#      include <strings.h>
#   endif
#endif

#undef VECMAC_VECTOR_TYPE 
#define VECMAC_VECTOR_TYPE 1
#include "vecmacP.h"
#define WANT_VECTP_ALIASES
#include "vectP.h" /* to be able to use vector library routines. */
#include "listP.h" /* to be able to use list library routines. */


#include "g3d.h"


#define VISUAL_PIRAT_TEST 0
#if VISUAL_PIRAT_TEST
#  include "Esimple.h"
#endif



struct pirat_leaf_t;


/*
  This is the type of the BBT front-end.  
 */
typedef struct pirat_front_end_t {
  /* Settable parameter:
     Stop searching when you find /find_max_num_objects/.
     Default is find_max_num_objects=INT_MAX
      */
  int                find_max_num_objects;
  /* Internal use only, may be inspected:
     Total number of points inserted into the tree.
     */
  int                num_of_inserted_points;
  /* Internal use only, may be inspected:
     Largest number of points per box.  This parameter is set at
     initialization.
     */
  int                max_num_of_points_per_box;
  /* Internal use only, may be inspected:
     Tolerance by which the root is increased when it is created.
     This parameter is set at initialization.
     */
  double             inflate_tol;
  /* Internal use only, may be inspected:
     Number of points found.
     */
  int                num_of_points_found;
  /* Internal use only:
     Size of the buffer below.
     */
  int                size_of_buffer;
  /* Internal use only, may be inspected:
     For each point found, this array records the entity found.
     */
  void             **entities_found;
  /* Internal use only:
     Counter for the iteration of the entities.
     */
  int                curr_entity;
  /* Internal use only:
     Storage optimization device.
     */
  void              *free_pts_array;
  /* Internal use only:
     The root of the tree.
     */
  struct pirat_leaf_t *root;
}               pirat_front_end_t;






/*
  Create a new PiRaT (point-in-range tree).
 */
pirat_front_end_t *
pirat_new_pirat(int max_num_of_points_per_box, double inflate_tol);


/*
  The function adds a point to the search tree.
 */
int
pirat_add_pt(pirat_front_end_t *pirat, void *object, G3D_loc_t *loc);


/*
   This function searches the tree for all the points which contained
   in the search range.  It returns TRUE if search was successful.
 */
int
pirat_search_range(pirat_front_end_t *pirat, G3D_box_t *range, int max_found);


void *
pirat_first_in_range(pirat_front_end_t *pirat);


void *
pirat_next_in_range(pirat_front_end_t *pirat);

void
pirat_destroy_pirat (pirat_front_end_t *);

#ifdef __cplusplus
}
#endif

#endif /* PIRAT_H */

