
#ifndef PIRATUTILS_H
#   define PIRATUTILS_H

#include <assert.h>



int
pirat_error(char *file, int line, int kind, char *explanation);


#define PIRAT_FATAL_ERROR   'f'
#define PIRAT_ERROR         'e'
#define PIRAT_WARNING       'w'




#define REPORT_WARNING(w) {pirat_error(__FILE__, __LINE__, PIRAT_WARNING, w);}

#define REPORT_ERROR(err) {pirat_error(__FILE__, __LINE__, PIRAT_ERROR, err);}

#define REPORT_FATAL_ERROR(err) {pirat_error(__FILE__, __LINE__, PIRAT_FATAL_ERROR, err);}



#define PIRAT_CHECK_NEW(ptr) \
   { if (ptr == NULL) { REPORT_FATAL_ERROR("Failed malloc"); } }


/* Ordinary loop */
#define FOR(_Counter_, _Up_)                                      \
   for (_Counter_ = 0; _Counter_ < _Up_; _Counter_++)


#ifndef min
#   define min(A,B) (((A) < (B)) ? (A) : (B))
#endif
#ifndef max
#   define max(A,B) (((A) > (B)) ? (A) : (B))
#endif
#ifndef min_of_3
#   define min_of_3(A,B,C)           \
          (((A) < (B))?              \
           (((C) < (A))? (C): (A)):  \
           (((C) < (B))? (C): (B))   \
           )
#   define max_of_3(A,B,C)           \
          (((A) > (B))?              \
           (((C) > (A))? (C): (A)):  \
           (((C) > (B))? (C): (B))   \
           )
#endif



#define PIRAT_NEW(TYPE) ((TYPE *) malloc(sizeof(TYPE)))

#define PIRAT_BCOPY(DEST, SRC, LEN) memcpy(DEST, SRC, LEN)

#define PIRAT_NEW_ARRAY(NELT, TYPE) \
                      ((TYPE *) calloc((NELT), (size_t) sizeof(TYPE)))

#define PIRAT_REALLOC_ARRAY(PTR, NEW_SIZE, TYPE) \
   ((TYPE *) realloc(PTR, (size_t)(NEW_SIZE * sizeof(TYPE))))

#define PIRAT_FREE(PTR) free(PTR)

#define PIRAT_DUPL_STRING(dest, src)                                      \
  {                                                                     \
    if (((dest) = PIRAT_NEW_ARRAY(strlen(src)+1, char)) == NULL) {        \
      PIRAT_CHECK_NEW(NULL);                                              \
    }                                                                   \
    strcpy((dest), (src));                                              \
  }




void
pirat_assert_failed(char *expr, char *file, int line);

#undef Assert
#ifdef STDC_HEADERS
#define Assert(EX)                                                        \
 ((##EX)                                                                      \
  ? ((void)0)                                                                 \
  : (void)pirat_assert_failed(# EX, __FILE__, __LINE__))
#else
#define Assert(EX)                                                        \
 ((EX)                                                                        \
  ? ((void)0)                                                                 \
  : (void)pirat_assert_failed("EX", __FILE__, __LINE__))
#endif

#endif /* PIRATUTILS_H  */
